/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.WGet;
import org.greenstone.gatherer.file.FileNode;
import org.greenstone.gatherer.gui.GLIButton;
import org.greenstone.gatherer.gui.Preferences;
import org.greenstone.gatherer.gui.URLField;
import org.greenstone.gatherer.util.StaticStrings;
import org.greenstone.gatherer.util.Utility;

/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1
 */
public class MirrorPane
    extends JPanel {

    private boolean download_button_enabled = false;
    private boolean ready = false;
    private JButton download_button;
    private JCheckBox higher_checkbox;
    private JCheckBox requisite_checkbox;
    private JCheckBox same_host_checkbox;
    private JScrollPane list_scroll;
    private JComboBox depth_combobox;
    private URLField url_field;
    private Vector depth_model;
    private WGet getter;

    public MirrorPane() {
	super();
	// Create a new wget thread
        getter = new WGet();
        getter.start();
	// And retrieve its list of pending jobs
	list_scroll = getter.getJobList();

	// Create
	JPanel edit_pane = new JPanel();
	JPanel details_pane = new JPanel();

	JPanel url_pane = new JPanel();
	JLabel url_label = new JLabel();
	Dictionary.registerText(url_label, "Mirroring.Source_URL");
	url_field = new URLField(Gatherer.config.getColor("coloring.editable_foreground", false), Gatherer.config.getColor("coloring.editable_background", false), Gatherer.config.getColor("coloring.error_foreground", false), Gatherer.config.getColor("coloring.error_background", false));
	Dictionary.registerTooltip(url_field, "Mirroring.Source_URL_Tooltip");

	depth_model = new Vector();
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(0, "Mirroring.Download_Depth.Zero"));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(1, String.valueOf(1)));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(2, String.valueOf(3)));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(3, String.valueOf(3)));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(4, String.valueOf(4)));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(5, String.valueOf(5)));
	/* @todo - add to dictionary */
	depth_model.add(new DepthEntry(-1, "Mirroring.Download_Depth.Unlimited"));
	JPanel depth_pane = new JPanel();
	JLabel depth_label = new JLabel();
	Dictionary.registerText(depth_label, "Mirroring.Download_Depth");
	depth_combobox = new JComboBox(depth_model);
	Dictionary.registerTooltip(depth_combobox, "Mirroring.Download_Depth_Tooltip");

	requisite_checkbox = new JCheckBox();
	Dictionary.registerText(requisite_checkbox, "Mirroring.Download_Hidden");

	higher_checkbox = new JCheckBox();
	Dictionary.registerText(higher_checkbox, "Mirroring.Higher_Directories");

	same_host_checkbox = new JCheckBox();
	same_host_checkbox.setSelected(true);
	Dictionary.registerText(same_host_checkbox, "Mirroring.Same_Host");

	JPanel button_pane = new JPanel();

	JButton preferences_button = new GLIButton("Mirroring.Further_Options");
	preferences_button.setMnemonic(KeyEvent.VK_P);

	download_button = new GLIButton();
	download_button.setEnabled(false);
	download_button.setMnemonic(KeyEvent.VK_D);
	Dictionary.registerBoth(download_button, "Mirroring.Download", "Mirroring.Download_Tooltip");

	// Connect
	download_button.addActionListener(new DownloadButtonListener());
	preferences_button.addActionListener(new PreferencesButtonActionListener());
	url_field.getDocument().addDocumentListener(new DownloadButtonEnabler());

	// Layout
	url_pane.setLayout(new BorderLayout(5,0));
	url_pane.add(url_label, BorderLayout.WEST);
	url_pane.add(url_field, BorderLayout.CENTER);

	depth_pane.setLayout(new GridLayout(1,3));
	depth_pane.add(depth_label);
	depth_pane.add(new JPanel());
	depth_pane.add(depth_combobox);

	details_pane.setLayout(new GridLayout(5,1,0,5));
	details_pane.add(url_pane);
	details_pane.add(depth_pane);
	details_pane.add(requisite_checkbox);
	details_pane.add(higher_checkbox);
	details_pane.add(same_host_checkbox);

	button_pane.setLayout(new GridLayout(1,3));
	button_pane.add(preferences_button);
	button_pane.add(new JPanel());
	button_pane.add(download_button);

	edit_pane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(2,0,0,0), BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(Dictionary.get("Mirroring.Download_Controls")), BorderFactory.createEmptyBorder(2,2,2,2))));
	edit_pane.setLayout(new BorderLayout());
	edit_pane.add(details_pane, BorderLayout.CENTER);
	edit_pane.add(button_pane, BorderLayout.SOUTH);

	setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	setLayout(new BorderLayout());
	add(edit_pane, BorderLayout.NORTH);
	add(list_scroll, BorderLayout.CENTER);
    }

    public void afterDisplay() {
	ready = true;
    }

    public void collectionChanged(boolean ready) {
    }

    /** This method is called whenever the Mirror pane is brought into focus and is a good time to display a warning message if WGet is not available or of an older, problematic, version.
     */
    public void gainFocus() {
	if(!ready) {
	    return;
	}
	// Lets see what warning message we should display, if any.
	String wget_version_str = Gatherer.config.getWGetVersion();
	if(wget_version_str.equals(StaticStrings.NO_WGET_STR)) {
	    // If there was no WGet available then downloading is disabled entirely
	    download_button_enabled = false;
	    // And we tell the user why.
	    /** @todo - add to dictionary */
	    JOptionPane.showMessageDialog(Gatherer.g_man, "Mirroring.No_WGet", "Mirroring.No_WGet_Title", JOptionPane.ERROR_MESSAGE);
	}
	else if(wget_version_str.equals(StaticStrings.WGET_OLD_STR)) {
	    // Downloading is enabled
	    download_button_enabled = true;
	    // But we display a preventable warning message about the path problems.
	    WarningDialog dialog = new WarningDialog("warning.OldWGet", false);
	    dialog.display();
	    dialog.dispose();
	    dialog = null;
	}
	// Otherwise version must be ok
	else {
	    download_button_enabled = true;
	}
	// It is also a good time to determine if the download should be enabled - ie if its allowed to be enabled and a valid URL is present in the field.
	download_button.setEnabled(download_button_enabled && !url_field.isEmpty() && url_field.validateURL());
    }

    public void setURL(String url) {
	url_field.setText(url);
    }


    static public FileNode getWebCacheMapping()
    {
  	if (Gatherer.config.get("workflow.mirror", true) || Gatherer.config.get("workflow.mirror", false)) {
	    /* @todo - add to dictionary */
  	    return new FileNode(Utility.getCacheDir(), "Mirroring.Mirror_Cache");
	}
	return null;
    }

    /** A DepthEntry contains a depth value, as an int, and a representitive text string. This allows for a more meaningful appearance than '0' for instance. */
    private class DepthEntry 
	implements Comparable {
	private int value;
	private String text;
	/** Default constructor. 
	 * @param value the depth value as an int
	 * @param text how this entry show represent itself as a String
	 */
	public DepthEntry(int value, String text) {
	    this.text = text;
	    this.value = value;
	}
	/** Determines the natural ordering of this DepthEntry and some other object
	 * @param object the Object to test for ordering
	 * @return >0 if the object is before this one, 0 if they are equal, or <0 if the object is after this one
	 */
	public int compareTo(Object object) {
	    // If the object is a DepthEntry then this is easy pesy.
	    if(object instanceof DepthEntry) {
		int target_value = ((DepthEntry) object).getValue();
		return value - target_value;
	    }
	    // If the object is a String, then we should try parsing an int from it
	    if(object instanceof String) {
		try {
		    int target_value = Integer.parseInt((String)object);
		    return value - target_value;
		}
		catch(Exception exception) {
		}
	    }
	    // Otherwise we'll try a string comparison between our text and the toString of the object
	    return text.compareTo(object.toString());
	}

	/** Determine if this DepthEntry is equal to some other object. Given that there is significant processing involved we push the resposiblility for this onto compareTo, so we only have the complex code once.
	 * @param object the Object to test for equality
	 * @return true if the object is equal to this depth entry, false otherwise
	 */
	public boolean equals(Object object) {
	    return (compareTo(object) == 0);
	}

	public int getValue() {
	    return value;
	}

	public String toString() {
	    return text;
	}
    }

    private class DownloadButtonEnabler
	implements DocumentListener {
	/** Gives notification that an attribute or set of attributes changed. */
	public void changedUpdate(DocumentEvent e) {
	    validateDownloadButton();
	}
        /** Gives notification that there was an insert into the document. */
	public void insertUpdate(DocumentEvent e) {
	    validateDownloadButton();
	}
        /** Gives notification that a portion of the document has been removed.	*/
	public void removeUpdate(DocumentEvent e) {
	    validateDownloadButton();
	} 
	private void validateDownloadButton() {
	    download_button.setEnabled(download_button_enabled && !url_field.isEmpty() && url_field.validateURL());
	}
    }

    private class DownloadButtonListener
	implements ActionListener {

	public void actionPerformed(ActionEvent event) {
	    // Retrieve the current url and confirm it is valid
	    String url_str = url_field.getText();
	    URL url = null;
	    try {
		url = new URL(url_str);
	    }
	    catch(MalformedURLException error) {
		JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("Mirroring.Invalid_URL"), Dictionary.get("Mirroring.Invalid_URL_Title"), JOptionPane.ERROR_MESSAGE);
	    }

	    // Otherwise construct a new download job
	    Object depth_object = depth_combobox.getSelectedItem();
	    int depth_value = 0;
	    if(depth_object instanceof DepthEntry) {
		depth_value = ((DepthEntry)depth_object).getValue();
	    }
	    else if(depth_object instanceof String) {
		String depth_string = (String) depth_object;
		try {
		    depth_value = Integer.parseInt(depth_string);
		    // Create a new entry for the purposes of comparison, and if needs be, addition
		    DepthEntry new_entry = new DepthEntry(depth_value, depth_string);
		    // Try to select the correct item from the combobox
		    depth_combobox.setSelectedItem(null);
		    depth_combobox.setSelectedItem(new_entry);
		    // We can see if that worked, and if it didn't add the entry
		    if(depth_combobox.getSelectedItem() == null) {
			depth_combobox.addItem(new_entry);
			depth_combobox.setSelectedItem(new_entry);
		    }
		    new_entry = null;
		}
		catch(Exception exception) {
		    /* @todo - add to dictionary */
		    JOptionPane.showMessageDialog(Gatherer.g_man, "Mirroring.Mirror_Depth.Invalid_Depth", "Mirroring.Mirror_Depth.Invalid_Depth_Title", JOptionPane.ERROR_MESSAGE);
		    return;
		}
		depth_string = null;
	    }
	    depth_object = null;
    
	    File destination_file = Utility.getCacheDir();
	    destination_file.mkdirs(); // If they aren't already
	    String destination_filename = destination_file.getAbsolutePath();
	    if(!destination_filename.endsWith(File.separator)) {
		destination_filename = destination_filename + File.separator;
	    }

	    if(url != null) {
		getter.newJob(!higher_checkbox.isSelected(), !same_host_checkbox.isSelected(), requisite_checkbox.isSelected(), url, depth_value, destination_filename);
	    }
	}
    }

    private class PreferencesButtonActionListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    new Preferences(Preferences.CONNECTION_PREFS);
	}
    }
}
